@@ -1,8 +1,9 @@
 #pragma once
 
-#include <regex>
 #include <string>
 
+#include <valijson/internal/regex.hpp>
+
 namespace valijson {
 namespace internal {
 namespace uri {
@@ -25,10 +26,10 @@ inline bool isUriAbsolute(const std::string &documentUri)
  * This function validates that the URI matches the RFC 8141 spec
  */
 inline bool isUrn(const std::string &documentUri) {
-  static const std::regex pattern(
+  static const internal::regex pattern(
       "^((urn)|(URN)):(?!urn:)([a-zA-Z0-9][a-zA-Z0-9-]{1,31})(:[-a-zA-Z0-9\\\\._~%!$&'()\\/*+,;=]+)+(\\?[-a-zA-Z0-9\\\\._~%!$&'()\\/*+,;:=]+){0,1}(#[-a-zA-Z0-9\\\\._~%!$&'()\\/*+,;:=]+){0,1}$");
 
-  return std::regex_match(documentUri, pattern);
+  return internal::regex_match(documentUri, pattern);
 }
 
 /**
